import React, { useCallback } from "react";
import { CustomTable } from "../../common/table/CustomTable";
import { Grid, Tooltip, Typography } from "@material-ui/core";
import { IOCStatusIcon } from "../IOCIcons";
import { noWrapText } from "../../common/Helper";
import { useRedirect } from "../../../hooks/Redirect";

const ownIocsColumns = [
  { id: "status", label: "Status", width: "8ch", textAlign: "center" },
  { id: "namingName", label: "IOC name", width: "15ch", sortable: false },
  { id: "host", label: "Host", width: "10ch", sortable: false },
  { id: "network", label: "Network", width: "10ch", sortable: false },
  { id: "sourceVersion", label: "Reference", width: "12ch", sortable: false }
];

const exploreIocsColumns = [
  { id: "status", label: "Status", width: "8ch", textAlign: "center" },
  { id: "namingName", label: "IOC name", width: "15ch", sortable: false },
  { id: "host", label: "Host", width: "10ch", sortable: false },
  { id: "network", label: "Network", width: "10ch", sortable: false },
  { id: "owner", label: "Maintainer", width: "13ch", sortable: false },
  { id: "sourceVersion", label: "Reference", width: "12ch", sortable: false }
];

const hostDetailsColumns = [
  { id: "status", label: "Status", width: "8ch", textAlign: "center" },
  { id: "namingName", label: "IOC name", width: "15ch", sortable: false },
  { id: "sourceVersion", label: "Reference", width: "12ch", sortable: false }
];

function createGitVersionField(version, shortVersion) {
  const versionText = shortVersion ?? version;
  return (
    <>
      {version ? (
        <Tooltip
          title={version}
          arrow
          enterDelay={400}
        >
          <Typography
            style={{
              fontFamily: "monospace",
              overflow: "hidden",
              textOverflow: "ellipsis",
              whiteSpace: "nowrap"
            }}
          >
            {versionText?.length > 10
              ? `${versionText.substring(0, 10)}...`
              : versionText}
          </Typography>
        </Tooltip>
      ) : (
        "---"
      )}
    </>
  );
}

function createTableRowForHostDetails(ioc) {
  const { id, name, namingName, activeDeployment, alert } = ioc;

  const sourceVersion = activeDeployment?.sourceVersion;
  const sourceVersionShort = activeDeployment?.sourceVersionShort;

  return {
    id: id,
    status: (
      <Grid
        container
        direction="column"
        justifyContent="center"
        alignItems="center"
      >
        <IOCStatusIcon ioc={ioc} />
      </Grid>
    ),
    namingName: noWrapText(namingName ?? name),
    sourceVersion: createGitVersionField(sourceVersion, sourceVersionShort),
    discrepancy: alert === "WARNING",
    inconsistentState: alert === "ERROR"
  };
}

function createTableRowForOwnIocs(ioc) {
  const { id, activeDeployment, namingName, alert } = ioc;

  const sourceVersion = activeDeployment?.sourceVersion;
  const sourceVersionShort = activeDeployment?.sourceVersionShort;

  return {
    id: id,
    status: (
      <Grid
        container
        direction="column"
        justifyContent="flex-start"
        alignItems="center"
      >
        <IOCStatusIcon ioc={ioc} />
      </Grid>
    ),
    namingName: noWrapText(namingName ?? "---"),
    sourceVersion: createGitVersionField(sourceVersion, sourceVersionShort),
    host: noWrapText(activeDeployment?.host.hostName || "---"),
    network: noWrapText(activeDeployment?.host.network || "---"),
    discrepancy: alert === "WARNING",
    inconsistentState: alert === "ERROR",
    iocNotDeployed: !ioc.activeDeployment
  };
}

function createTableRowForExploreIocs(ioc) {
  const { id, namingName, owner, activeDeployment, alert, host } = ioc;

  const sourceVersion = activeDeployment?.sourceVersion;
  const sourceVersionShort = activeDeployment?.sourceVersionShort;

  return {
    id: id,
    status: (
      <Grid
        container
        direction="column"
        justifyContent="center"
        alignItems="center"
      >
        <IOCStatusIcon ioc={ioc} />
      </Grid>
    ),
    namingName: noWrapText(namingName ?? "---"),
    sourceVersion: createGitVersionField(sourceVersion, sourceVersionShort),
    host: noWrapText(host?.hostName || "---"),
    network: noWrapText(host?.network || "---"),
    owner: noWrapText(owner || "---"),
    discrepancy: alert === "WARNING",
    inconsistentState: alert === "ERROR",
    iocNotDeployed: !ioc.activeDeployment
  };
}

export function IOCTable({
  iocs,
  rowType = "own",
  totalCount,
  lazyParams,
  setLazyParams,
  rowsPerPage,
  loading,
  paginator
}) {
  const tableTypeSpecifics = {
    own: [ownIocsColumns, createTableRowForOwnIocs],
    host: [hostDetailsColumns, createTableRowForHostDetails],
    explore: [exploreIocsColumns, createTableRowForExploreIocs]
  };

  const redirect = useRedirect();
  const [columns, createRow] = tableTypeSpecifics[rowType];
  const rows = iocs.map((ioc) => createRow(ioc));

  const itemLink = (item) => `/iocs/${item.id}`;

  const onRowClicked = useCallback(
    (id, item) => {
      redirect(itemLink(item));
    },
    [redirect]
  );

  return (
    <CustomTable
      columns={columns}
      totalCount={totalCount}
      rows={rows}
      handleRowClick={onRowClicked}
      itemLink={itemLink}
      lazyParams={lazyParams}
      setLazyParams={setLazyParams}
      rowsPerPage={rowsPerPage}
      loading={loading}
      paginator={paginator}
    />
  );
}
